@@ -5,7 +5,7 @@
  * Author:   Georg Fleischmann
  *
  * created:  1995-08-10
- * modified: 2005-01-04
+ * modified: 2005-01-13
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -114,8 +114,6 @@
                                                          : vhfHomeDirectory());
     if (!path)
         return vhfHomeDirectory();
-    if (!path)	// Apple Check to find nil directories
-        NSLog(@"[App currentDirectory]: path == nil, NSHomeDirectory() = '%@'", NSHomeDirectory());
     return path;
 }
 
