@@ -5,7 +5,7 @@
  * Author: Georg Fleischmann
  *
  * Created:  1996-02-09
- * Modified: 2005-01-11
+ * Modified: 2005-05-06 (save printinfo)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -1146,10 +1146,11 @@ static NSMutableArray *zoneList = nil;
     NSAutoreleasePool	*pool0 = [NSAutoreleasePool new], *pool;
     NSString		*pStr=nil, *npStr=nil, *pExt=nil, *mStr, *paperName = [pi paperName];
     NSMutableArray	*ml, *mlist=[NSMutableArray array], *layerList;
-    NSRect		mBounds, pageRect = NSZeroRect, newPageRect = NSZeroRect, oldFrame;
+    NSRect		mBounds, pageRect = NSZeroRect, newPageRect = NSZeroRect, oldFrame, visibleRect;
     NSPoint		mPosition, mOffset, lr, ll, ur, ul, center, dm, pOffset = {55.0, 55.0};
     int			i, orientation = [pi orientation];
     DocView		*docView = [self documentView];
+    float		width, height, scale = [docView scaleFactor];
 
     if ([piDict objectForKey:NSPrintSavePath])
     {
@@ -1173,15 +1174,19 @@ static NSMutableArray *zoneList = nil;
     //[pi setPaperName:[NSString stringWithFormat:@"Other"]];
     [pi setOrientation:orientation];
 
-    {   float	width, height, scale = [docView scaleFactor];
-
-        oldFrame = [docView frame];
-        width = oldFrame.size.width;
-        height = oldFrame.size.height;
-        width  += (pOffset.x*2.0) * scale;
-        height  += (pOffset.y*2.0) * scale;
-        [docView setFrameSize:NSMakeSize(width, height)];
-    }
+    /* set scale to 1.0
+     * and add pOffset to frameSize
+     */
+    scale = [docView scaleFactor];
+    visibleRect = [docView visibleRect]; // needed later to calc the center to scroll back
+    oldFrame = [docView frame];
+    width  = oldFrame.size.width  / scale;
+    height = oldFrame.size.height / scale;
+    width  += (pOffset.x*2.0); // * scale
+    height += (pOffset.y*2.0); // * scale
+    [window disableFlushWindow];
+    [docView scaleUnitSquareToSize:NSMakeSize(1.0/scale, 1.0/scale)];
+    [docView setFrameSize:NSMakeSize(width, height)];
 
     /* up left - get separation mark */
     mStr = [NSString stringWithFormat:@"%@/%@/leftup.cenon", [[NSBundle mainBundle] resourcePath], CROPMARK_FOLDER];
@@ -1528,17 +1533,39 @@ static NSMutableArray *zoneList = nil;
         }
     }
 
+    /* scale view back to old scale */
+    [docView scaleUnitSquareToSize:NSMakeSize(scale, scale)];
     [docView setFrameSize:oldFrame.size];
+    /* scroll docView to old position */
+    [docView scrollPoint:visibleRect.origin];
     [docView drawAndDisplay];
 
+    [window enableFlushWindow];
+
     [pool0 release];
     [[self documentView] setSeparationColor:nil];
 }
 
+/*
+ * modified: 2005-05-06
+ */
 - (void)printDocument:sender
 {   NSPrintOperation	*op;
     NSPrintPanel	*pp;
     NSPrintInfo		*pi;
+    DocView		*docView = [self documentView];
+    NSRect		visibleRect, oldFrame;
+    float		width, height, scale = [docView scaleFactor];
+
+    /* set scale to 1.0 */
+    scale = [docView scaleFactor];
+    visibleRect = [docView visibleRect]; // needed later to calc the center to scroll back
+    oldFrame = [docView frame];
+    width  = oldFrame.size.width  / scale;
+    height = oldFrame.size.height / scale;
+    [window disableFlushWindow];
+    [docView scaleUnitSquareToSize:NSMakeSize(1.0/scale, 1.0/scale)];
+    [docView setFrameSize:NSMakeSize(width, height)];
 
     /* FIXME: GNUstep printing needs to be fixed for the code below */
 #ifdef GNUSTEP_BASE_VERSION
@@ -1572,9 +1599,18 @@ static NSMutableArray *zoneList = nil;
         [self printSeparation:op];
     }
     pi = [op printInfo];
-    if ([pi printer])
-        [printInfo setPrinter:[pi printer]];	// remember printer for the next time !
+    [printInfo release];
+    printInfo = [pi retain];	// remember printInfo values for the next time ! (2005-05-06)
 #endif
+
+    /* scale view back to old scale */
+    [docView scaleUnitSquareToSize:NSMakeSize(scale, scale)];
+    [docView setFrameSize:oldFrame.size];
+    /* scroll docView to old position */
+    [docView scrollPoint:visibleRect.origin];
+    [docView drawAndDisplay];
+
+    [window enableFlushWindow];
 }
 
 /*
