@@ -1,11 +1,11 @@
 /* VRectangle.m
  * 2-D Rectangle object
  *
- * Copyright (C) 1996-2004 by vhf interservice GmbH
+ * Copyright (C) 1996-2005 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-09-17
- * modified: 2004-02-28
+ * modified: 2005-04-25 (clean up)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -347,11 +347,11 @@
 - (VPath*)pathRepresentation
 {   VPath		*pathG = [VPath path];
     VLine		*line;
-    VArc			*arc;
+    VArc		*arc;
     int			i, j;
     BOOL		ccw = [self isDirectionCCW];
     NSPoint		p0, p1, c, pts[5];
-    float		a = (ccw) ? 90.0 : -90.0;
+    float		a = (ccw) ? 90.0 : -90.0, r = radius;
 
     [pathG setDirectionCCW:ccw];
 
@@ -359,20 +359,22 @@
     pts[1].x = origin.x + size.x; pts[1].y = origin.y;
     pts[2].x = origin.x + size.x; pts[2].y = origin.y + size.y;
     pts[3].x = origin.x; pts[3].y = origin.y + size.y;
+    if (size.x < r*2.0 || size.y < r*2.0)
+        r = 0.0;
 
     for ( i=(ccw)?0:4; (ccw)?i<4:i; (ccw)?i++:i-- )
     {
         j = (ccw) ? i+1 : i-1;
-        p0.x = pts[i].x + ((pts[i].x<pts[j].x) ? radius : ((pts[i].x>pts[j].x) ? -radius : 0));
-        p0.y = pts[i].y + ((pts[i].y<pts[j].y) ? radius : ((pts[i].y>pts[j].y) ? -radius : 0));
-        p1.x = pts[j].x + ((pts[i].x<pts[j].x) ? -radius : ((pts[i].x>pts[j].x) ? radius : 0));
-        p1.y = pts[j].y + ((pts[i].y<pts[j].y) ? -radius : ((pts[i].y>pts[j].y) ? radius : 0));
+        p0.x = pts[i].x + ((pts[i].x<pts[j].x) ? r : ((pts[i].x>pts[j].x) ? -r : 0));
+        p0.y = pts[i].y + ((pts[i].y<pts[j].y) ? r : ((pts[i].y>pts[j].y) ? -r : 0));
+        p1.x = pts[j].x + ((pts[i].x<pts[j].x) ? -r : ((pts[i].x>pts[j].x) ? r : 0));
+        p1.y = pts[j].y + ((pts[i].y<pts[j].y) ? -r : ((pts[i].y>pts[j].y) ? r : 0));
         line = [VLine line];
         [line setVertices:p0 :p1];
         [line setColor:color];
         [[pathG list] addObject:line];
 
-        if ( radius )
+        if ( r )
         {
             c.x = pts[j].x + ((pts[j].x>pts[0].x) ? -radius : radius);
             c.y = pts[j].y + ((pts[j].y>pts[0].y) ? -radius : radius);
@@ -513,7 +515,7 @@
 {
     if (!flag)
         selectedKnob = -1;
-    return [super setSelected:flag];
+    [super setSelected:flag];
 }
 
 - (void)scale:(float)x :(float)y withCenter:(NSPoint)cp
@@ -644,7 +646,9 @@
         }
     }
     else if (filled <= 1)
-        return [[self pathRepresentation] drawWithPrincipal:principal];
+    {   [[self pathRepresentation] drawWithPrincipal:principal];
+        return;
+    }
     else // filled is 2 or 3 or 4
     {   VPath	*pathRep = [self pathRepresentation];
 
@@ -1259,13 +1263,13 @@
  * modified: 2004-02-28
  */
 - uniteWith:(id)ug
-{   int			i, j = 0, k, l, endIx=0, uStartIs[100], startI, listCnt, uStartIsCnt = 0, uListCnt;
-    int			sPairsCnt = 0, ePairsCnt = 0, sPairsCnts[50], ePairsCnts[50];
-    int			removedFromUg = 0, removedFromNg = 0, sCnt=0, eCnt=0, startIs[100], endIs[100];
+{   int			i, j = 0, k, l, endIx=0, uStartIs[1000], startI, listCnt, uStartIsCnt = 0, uListCnt;
+    int			sPairsCnt = 0, ePairsCnt = 0, sPairsCnts[500], ePairsCnts[500];
+    int			removedFromUg = 0, removedFromNg = 0, sCnt=0, eCnt=0, startIs[1000], endIs[1000];
     float		tol = (10.0*TOLERANCE);
     VPath		*ng;
     NSMutableArray	*splitListG, *splitListUg;
-    NSPoint		p, startPts[100], endPts[100];   // start/end point of removed graphic(s)
+    NSPoint		p, startPts[1000], endPts[1000];   // start/end point of removed graphic(s)
     BOOL		first = YES, removing = NO;
     NSAutoreleasePool	*pool;
 
@@ -2161,15 +2165,22 @@
 
     [plist setObject:propertyListFromNSPoint(origin) forKey:@"origin"];
     [plist setObject:propertyListFromNSPoint(size) forKey:@"size"];
-    [plist setObject:propertyListFromFloat(radius) forKey:@"radius"];
-    [plist setObject:propertyListFromFloat(rotAngle) forKey:@"rotAngle"];
+    if (radius)
+        [plist setObject:propertyListFromFloat(radius) forKey:@"radius"];
+    if (rotAngle)
+        [plist setObject:propertyListFromFloat(rotAngle) forKey:@"rotAngle"];
     if (filled)
         [plist setObject:propertyListFromInt(filled) forKey:@"filled"];
-    [plist setObject:propertyListFromNSColor(fillColor) forKey:@"fillColor"];
-    [plist setObject:propertyListFromNSColor(endColor) forKey:@"endColor"];
-    [plist setObject:propertyListFromFloat(graduateAngle) forKey:@"graduateAngle"];
-    [plist setObject:propertyListFromFloat(stepWidth) forKey:@"stepWidth"];
-    [plist setObject:propertyListFromNSPoint(radialCenter) forKey:@"radialCenter"];
+    if (fillColor != [NSColor blackColor])
+        [plist setObject:propertyListFromNSColor(fillColor) forKey:@"fillColor"];
+    if (endColor != [NSColor blackColor])
+        [plist setObject:propertyListFromNSColor(endColor) forKey:@"endColor"];
+    if (graduateAngle)
+        [plist setObject:propertyListFromFloat(graduateAngle) forKey:@"graduateAngle"];
+    if (stepWidth != 7)
+        [plist setObject:propertyListFromFloat(stepWidth) forKey:@"stepWidth"];
+    if (radialCenter.x != 0.5 && radialCenter.y != 0.5)
+        [plist setObject:propertyListFromNSPoint(radialCenter) forKey:@"radialCenter"];
     return plist;
 }
 - (id)initFromPropertyList:(id)plist inDirectory:(NSString *)directory
@@ -2187,8 +2198,12 @@
     if (!(endColor = colorFromPropertyList([plist objectForKey:@"endColor"], [self zone])))
         [self setEndColor:[NSColor blackColor]];
     graduateAngle = [plist floatForKey:@"graduateAngle"];
-    stepWidth = [plist floatForKey:@"stepWidth"];
-    radialCenter = pointFromPropertyList([plist objectForKey:@"radialCenter"]);
+    if ( !(stepWidth = [plist floatForKey:@"stepWidth"]))
+        stepWidth = 7.0;	// default;
+    if ([plist objectForKey:@"radialCenter"])
+        radialCenter = pointFromPropertyList([plist objectForKey:@"radialCenter"]);
+    else
+        radialCenter = NSMakePoint(0.5, 0.5);	// default
     [self setParameter];
     graduateDirty = YES;
     graduateList = nil;
