@@ -1,11 +1,11 @@
 /* VText.m
  * 2-D Text object
  *
- * Copyright (C) 1996-2003 by vhf interservice GmbH
+ * Copyright (C) 1996-2005 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-02-15
- * modified: 2003-06-26
+ * modified: 2005-06-10 (setColor...)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -154,6 +154,7 @@ static void initClassVars()
     endColor = [[NSColor blackColor] retain];
     graduateAngle = 0.0;
     stepWidth = 7.0;
+    radialCenter = NSMakePoint(0.5, 0.5);
     //lineHeight = [[drawText font] pointSize];
     return [super init];
 }
@@ -181,11 +182,30 @@ static void initClassVars()
 - (BOOL)filled			{ return filled; }
 - (void)setFilled:(BOOL)flag	{ filled = flag; dirty = YES; }
 
+/* modified: 2005-06-10
+ */
+- (void)setColor:(NSColor *)col
+{
+    [self setFillColor:col];
+}
+
+/* modified: 2005-06-10
+ */
 - (void)setFillColor:(NSColor*)col
 {
+    if (attributedString)
+    {   NSTextStorage	*ts = [drawText textStorage];
+
+        [ts beginEditing];
+        [ts setAttributedString:attributedString];
+        [ts endEditing];
+        [drawText setTextColor:col];
+        [drawText setFrame:bounds];
+        [self setAttributedString:[ts attributedSubstringFromRange:NSMakeRange(0, [ts length])]];
+    }
+    [super setColor:col];
     if (fillColor) [fillColor release];
     fillColor = [col retain];
-    [self setColor:fillColor]; // copy else redo kill
     dirty = YES;
 }
 - (NSColor*)fillColor { return fillColor; }
@@ -201,24 +221,12 @@ static void initClassVars()
 - (void)setGraduateAngle:(float)a	{ graduateAngle = a; dirty = YES; }
 - (float)graduateAngle			{ return graduateAngle; }
 
+- (void)setRadialCenter:(NSPoint)rc	{ radialCenter = rc; dirty = YES; }
+- (NSPoint)radialCenter			{ return radialCenter; }
+
 - (void)setStepWidth:(float)sw	{ stepWidth = sw; dirty = YES; }
 - (float)stepWidth		{ return stepWidth; }
 
-- (void)setColor:(NSColor *)col
-{
-    if (attributedString)
-    {   NSTextStorage	*ts = [drawText textStorage];
-
-        [ts beginEditing];
-        [ts setAttributedString:attributedString];
-        [ts endEditing];
-        [drawText setTextColor:col];
-        [drawText setFrame:bounds];
-        [self setAttributedString:[ts attributedSubstringFromRange:NSMakeRange(0, [ts length])]];
-    }
-    [super setColor:col];
-}
-
 - (void)setTextPath:tPath	{ textPath = tPath; }
 
 - (NSString*)string
@@ -289,7 +297,7 @@ static void initClassVars()
         font = [aFont retain];
     }
     if (aColor)
-        [super setColor:aColor];
+        [self setColor:aColor];	// 2005-06-10
     [self setString:string lineHeight:lineHeight];
     dirty = YES;
 }
@@ -338,6 +346,17 @@ static void initClassVars()
     }
 }
 
+- (void)replaceCharactersInRange:(NSRange)range withString:(NSString*)string
+{
+    if (string)
+    {
+        [self makeMutableAttributedString];
+        [attributedString replaceCharactersInRange:range withString:string];
+        if ( [self fitHorizontal] )
+            [self kernToFitHorizontal];
+    }
+}
+
 - (void)makeMutableAttributedString
 {
     if ( ![attributedString isKindOfClass:[NSMutableAttributedString class]] )
@@ -374,8 +393,8 @@ static void initClassVars()
         //[drawText replaceCharactersInRange:NSMakeRange(0, [ts length]) withRTF:data];
         //[self setAttributedString:[ts attributedSubstringFromRange:NSMakeRange(0, [ts length])]];
 
-        [self setAttributedString:[[NSAttributedString alloc] initWithRTF:data
-                                                       documentAttributes:NULL]];
+        [self setAttributedString:[[[NSAttributedString alloc] initWithRTF:data
+                                                        documentAttributes:NULL] autorelease]];
     }
     else
         [self setAttributedString:[[NSAttributedString new] autorelease]];
@@ -1682,6 +1701,7 @@ static void initClassVars()
     [aCoder encodeObject:fillColor];
     [aCoder encodeObject:endColor];
     [aCoder encodeValuesOfObjCTypes:"ff", &graduateAngle, &stepWidth];
+    [aCoder encodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
 }
 - (id)initWithCoder:(NSCoder *)aDecoder
 {   int	version;
@@ -1716,6 +1736,7 @@ static void initClassVars()
     {   fillColor = [[aDecoder decodeObject] retain];
         endColor = [[aDecoder decodeObject] retain];
         [aDecoder decodeValuesOfObjCTypes:"ff", &graduateAngle , &stepWidth];
+        [aDecoder decodeValuesOfObjCTypes:"{NSPoint=ff}", &radialCenter];
     }
     if ( version < 1 )
     {   float	f;
@@ -1740,10 +1761,17 @@ static void initClassVars()
     if (isSerialNumber) [plist setObject:@"YES" forKey:@"isSerialNumber"];
     if (fitHorizontal) [plist setObject:@"YES" forKey:@"fitHorizontal"];
     if (centerVertical) [plist setObject:@"YES" forKey:@"centerVertical"];
-    [plist setObject:propertyListFromNSColor(fillColor) forKey:@"fillColor"];
-    [plist setObject:propertyListFromNSColor(endColor) forKey:@"endColor"];
-    [plist setObject:propertyListFromFloat(graduateAngle) forKey:@"graduateAngle"];
-    [plist setObject:propertyListFromFloat(stepWidth) forKey:@"stepWidth"];
+
+    if (fillColor != [NSColor blackColor])
+        [plist setObject:propertyListFromNSColor(fillColor) forKey:@"fillColor"];
+    if (endColor != [NSColor blackColor])
+        [plist setObject:propertyListFromNSColor(endColor) forKey:@"endColor"];
+    if (graduateAngle)
+        [plist setObject:propertyListFromFloat(graduateAngle) forKey:@"graduateAngle"];
+    if (stepWidth != 7)
+        [plist setObject:propertyListFromFloat(stepWidth) forKey:@"stepWidth"];
+    if (radialCenter.x != 0.5 && radialCenter.y != 0.5)
+        [plist setObject:propertyListFromNSPoint(radialCenter) forKey:@"radialCenter"];
 
     return plist;
 }
@@ -1759,12 +1787,18 @@ static void initClassVars()
     fitHorizontal = ([plist objectForKey:@"fitHorizontal"] ? YES : NO);
     centerVertical = ([plist objectForKey:@"centerVertical"] ? YES : NO);
     [self font];
+
     if (!(fillColor = colorFromPropertyList([plist objectForKey:@"fillColor"], [self zone])))
         [self setFillColor:[color copy]];
     if (!(endColor = colorFromPropertyList([plist objectForKey:@"endColor"], [self zone])))
         [self setEndColor:[NSColor blackColor]];
     graduateAngle = [plist floatForKey:@"graduateAngle"];
-    stepWidth = [plist floatForKey:@"stepWidth"];
+    if ( !(stepWidth = [plist floatForKey:@"stepWidth"]))
+        stepWidth = 7.0;	// default;
+    if ([plist objectForKey:@"radialCenter"])
+        radialCenter = pointFromPropertyList([plist objectForKey:@"radialCenter"]);
+    else
+        radialCenter = NSMakePoint(0.5, 0.5);	// default
     return self;
 }
 
